<?php
// $Id: reptag_process.inc,v 1.22 2007/01/03 23:31:57 profix898 Exp $

require_once (drupal_get_path('module', 'reptag').'/regtag_helper.inc');

define('REPTAG_PROCALL',      0);
define('REPTAG_PROCCACHE',    1);
define('REPTAG_PROCDYNAMIC',  2);

define('REPTAG_NODEFULL',     1);
define('REPTAG_NODETEASER',   2);
define('REPTAG_COMMENT',      3);
define('REPTAG_USERTEXT',     4);

/**
 * Simple (wrapper) API function for text replacements
 * (You can use reptag_replace() to process a piece of text instead
 *  of calling reptag_process() directly)
 *
 * @param $text
 *   The text/content to be processed.
 * @param $procall
 *   By default only static tags are replaced using reptag_replace(),
 *   set $procall = TRUE to use all tags (static + dynamic) instead. 
 */
function reptag_replace($text, $procall = FALSE) {
  global $user;
  
  $param = new StdClass();
  $param->uid = $user->uid;
  $param->text = $text;
  reptag_process($param, REPTAG_USERTEXT, $procall ? REPTAG_PROCALL : REPTAG_PROCCACHE);
  
  return $param->text;
}

/**
 * Perform the actual reptag processing
 * (This function does all the filter/replacement stuff)
 *
 * @param $content
 *   An array containing the content to process.    
 * @param $type
 *   An integer to set the content type (e.g. REPTAG_NODETEASER, ...).
 * @param $proc
 *   An integer to set the processing mode (e.g. REPTAG_PROCALL, ...).
 *
 *   See DEVELOPER.TXT for detailed information  
 */
function reptag_process(&$content, $type = 0, $proc = REPTAG_PROCALL) {
  // check permission
  if (!user_access('invoke replacements')) {
    return;
  }
  // select text to filter
  switch ($type) {
    case REPTAG_NODEFULL:
      $text = &$content->body;
      break;
    case REPTAG_NODETEASER:
      $text = &$content->teaser;
      break;
    case REPTAG_COMMENT:
      $text = &$content->comment;
      break;
    case REPTAG_USERTEXT:
      $text = &$content->text;
      break;
    default:
      return;
  }
      
  // include module (.tags) ...
  $reptag_modules = array();
  $files = file_scan_directory(drupal_get_path('module', 'reptag'), '\.tags$');
  foreach ($files as $filename => $file) {
    if (variable_get('reptag_module_'.$file->name, 0)) {
      include_once $filename;
      $reptag_modules[$file->name] = $file->name;
    }
  }
  // ... and other modules (hook_reptag) ...
  $modules = module_implements('reptag');
  foreach($modules as $module) {
    if (variable_get('reptag_module_'.$module, 0)) {
      $reptag_modules[$module] = $module;
      $reptags_info[$module] = module_invoke($module, 'reptag', 'info');
    }
  }
  
  // ... depending on processing mode
  foreach ($reptag_modules as $module) {
    switch ($proc) {
      case REPTAG_PROCCACHE:
        if (!is_array($reptags_info[$module])) {
          unset($reptag_modules[$module]);
        }
        elseif (!$reptags_info[$module][1]) {
          unset($reptag_modules[$module]);
        }
        break;
      case REPTAG_PROCDYNAMIC:
        if (is_array($reptags_info[$module]) && $reptags_info[$module][1]) {
          unset($reptag_modules[$module]);
        }
        break;
      default:
      case REPTAG_PROCALL:
    }
  }
  
  if ($proc == REPTAG_PROCDYNAMIC && empty($reptag_modules)) {
    return;
  }
  
  // roles associated with current node
  $content_roles = _reptag_user_get_roles($content->uid);

  // load exclude tags
  $exclude_tags = array();
  foreach ($content_roles as $rid) {
    $rid_tags = _reptag_exclude_loadtags($rid);
    $exclude_tags = array_merge($exclude_tags, $rid_tags);
  }
  
  // load side-wide and user tags from tables
  if ($proc != REPTAG_PROCDYNAMIC) {
  
    $iid = array(0, $content->uid);
    $reptag_table = _reptag_table_load($iid);
    
    // remove all html tags for plain text roles
    $plainrep_roles = unserialize(variable_get('reptag_module_plainrep_roles', serialize(array())));
    if (array_intersect($plainrep_roles, $content_roles)) {
      foreach ($reptag_table as $tag => $repl) {
        $reptag_table[$tag] = check_plain(preg_replace('#<[\/\!]*?[^<>]*?>#si', '', $repl));
      }
    }
    
    // filter exclude_tags from reptag_table
    if (!empty($exclude_tags)) {
      foreach ($exclude_tags as $key) {
        $key = '#\\'.strtoupper(substr($key, 0, -1)).'\$#si'; 
        unset($reptag_table[$key]);
      }
    }
    
  }
  
  // load module variables (according to permissions)
  if (!empty($reptag_modules)) {
    foreach ($reptag_modules as $module) {
      $module_roles = unserialize(variable_get('reptag_module_'.$module.'_roles', serialize(array())));
      if (array_intersect($module_roles, $content_roles) || $content->uid == 1) {
        $init_func = 'reptag_'.$module.'_init';
        if (function_exists($init_func)) {
          if (!$reptags[$module] = $init_func($content)) {
            unset($reptags[$module]);
          }
          else {
            $require_func = 'reptag_'.$module.'_require';
            if (function_exists($require_func)) {
              if (!$require_func()) {
                unset($reptags[$module]);
              }
            }
          }
        }
        if (module_hook($module, 'reptag')) {
          if (!$reptags[$module] = module_invoke($module, 'reptag', 'init', $content)) {
            unset($reptags[$module]);
          }
        }
      }
    }
  }
  
  // filter exclude_tags from reptags
  $reptags = _reptag_filter_exclude($reptags, $exclude_tags);
  
  // debug
  if (variable_get('reptag_debug', 0)) {
    drupal_set_message('reptags: '.nl2br(htmlspecialchars(print_r($reptags, true))), 'error');
    drupal_set_message('reptag_table: '.nl2br(htmlspecialchars(print_r($reptag_table, true))), 'error');
    drupal_set_message('node: '.nl2br(htmlspecialchars(print_r($content, true))), 'error');
  }
  
  // process table tags
  if ($proc != REPTAG_PROCDYNAMIC) {
    $text = preg_replace(array_keys($reptag_table), array_values($reptag_table), $text);
  }
  
  // process module tags
  if (!empty($reptags)) {
    foreach ($reptags as $module => $tags) {
      $default = TRUE;
      $process_func = 'reptag_'.$module.'_process';
      if (function_exists($process_func)) {
        $text = $process_func($text, $tags, $content);
        $default = FALSE;
      }
      if (module_hook($module, 'reptag_process')) {
        $text = module_invoke($module, 'reptag_process', $text, $tags, $content);
        $default = FALSE;
      }
      if ($default) {
        $text = preg_replace(array_keys($tags), array_values($tags), $text);
      }
    }
  }
  
  // validate on preview
  if ($content->in_preview && variable_get('reptag_validate', 1)) {
    _reptag_validate_xml($text);
  }
}

/**
 * function _reptag_filter_exclude().
 */
function _reptag_filter_exclude($reptags, $exclude_tags) {
  if (!empty($reptags) && !empty($exclude_tags)) {
    foreach ($reptags as $module => $tags) {
      foreach($tags as $tag => $repl) {
        foreach($exclude_tags as $exclude_tag) {
          if (!empty($tag) && !empty($exclude_tag)) {
            if (stristr($tag, $exclude_tag)) {
              unset($reptags[$module][$tag]);
            }
          }
        }
      }
    }
  }
  
  return $reptags; 
}

/**
 * function _reptag_validate_xml().
 * (check current content being well-formed x(ht)ml)
 */
function _reptag_validate_xml($xml) {
  $version = phpversion();
  if (version_compare($version, '4.3.0', '>=')) {
    if (version_compare($version, '5.0.0', '>=') && class_exists('DOMDocument')) {
      // PHP 5.0.0+
      
      // extract html tags
      preg_match_all('/<(\/{0,1}[\w]+)[^>]*>/', $xml, $matches);
      $matches = $matches[0];
      
$pretag = <<< PRE
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title></title>
</head>
<body>
PRE;
      
$posttag = <<< POST
</body>
</html>
POST;
      
      $xmlhtml = $pretag.implode('', $matches).$posttag;
      
      $dom = new DOMDocument;
      $status = ini_set('html_errors', FALSE);
      $dom->loadHTML($xmlhtml);
      if (!$dom->validate()) {
        ini_set('html_errors', $status);
        return FALSE;
      }
      ini_set('html_errors', $status);

    }
    else {
      // PHP 4.3.0+
    
      if (!function_exists('xml_parser_create')) {
        return TRUE;
      }
      
      $xmlhtml = '<body>'.$xml.'</body>';
      $xml_parser = xml_parser_create();      
       if (!xml_parse($xml_parser, $xmlhtml)) {
          drupal_set_message('XML error: '.xml_error_string(xml_get_error_code($xml_parser)).' at line '.xml_get_current_line_number($xml_parser), 'error');
          return FALSE;
       }
      xml_parser_free($xml_parser);
    
    }
  }

  return TRUE;
}

?>
